# Generated by Django 3.2.21 on 2023-09-22 10:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0036_enterprisesubsidybudget_subsidy_access_policy_display_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterpriseenrollment',
            name='consent_granted',
            field=models.BooleanField(default=None, null=True),
        ),
    ]
